/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.lock;

import EDU.oswego.cs.dl.util.concurrent.Sync;

/**
 * Transaction isolation level of Read_Committed. It prevents dirty read only.
 * <p> Dirty read allows (t1) write and then (t2) read within two separate threads, all without
 * transaction commit. </p>
 *
 * @version $Revision: 1.1.2.1 $
 */
public class LockStrategyReadCommitted implements LockStrategy
{
   private NonBlockingWriterLock lock_;

   public LockStrategyReadCommitted()
   {
      lock_ = new NonBlockingWriterLock();
   }

   /**
    * @see org.jboss.cache.LockStrategy#readLock()
    */
   public Sync readLock()
   {
      return lock_.readLock();
   }

   /**
    * @see org.jboss.cache.LockStrategy#upgradeLockAttempt(long)
    */
   public Sync upgradeLockAttempt(long msecs)
   {
      return lock_.upgradeLockAttempt(msecs);
   }

   /**
    * @see org.jboss.cache.LockStrategy#writeLock()
    */
   public Sync writeLock()
   {
      return lock_.writeLock();
   }
}
