/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.lock;

import EDU.oswego.cs.dl.util.concurrent.FIFOSemaphore;
import EDU.oswego.cs.dl.util.concurrent.NullSync;
import EDU.oswego.cs.dl.util.concurrent.Sync;

/**
 * Transaction isolation level of Serializable. It prevents none of the dirty read,
 * non-repeatable read, or phantom read.
 *
 * @version $Revision: 1.1.2.1 $
 */
public class LockStrategyReadUncommitted implements LockStrategy
{
   private int permits_;
   private FIFOSemaphore wLock_;
   private NullSync rLock_; // Null lock will always return true

   public LockStrategyReadUncommitted()
   {
      wLock_ = new FIFOSemaphore(1);
      rLock_ = new NullSync();
   }

   /**
    * @see org.jboss.cache.LockStrategy#readLock()
    */
   public Sync readLock()
   {
      return rLock_;
   }

   /**
    * @see org.jboss.cache.LockStrategy#upgradeLockAttempt(long)
    */
   public Sync upgradeLockAttempt(long msecs)
   {
      return wLock_;
   }

   /**
    * @see org.jboss.cache.LockStrategy#writeLock()
    */
   public Sync writeLock()
   {
      return wLock_;
   }
}
