package org.jboss.cache.transaction;

import org.jboss.logging.Logger;

import javax.transaction.*;
import javax.transaction.xa.XAResource;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author bela
 * @version $Revision: 1.12.2.4 $
 *          Date: May 15, 2003
 *          Time: 4:20:17 PM
 */
public class DummyTransaction implements Transaction
{
   int status = Status.STATUS_UNKNOWN;
   Logger logger_ = Logger.getLogger(DummyTransaction.class);
   DummyTransactionManager tm_;

   /**
    * List<Synchronization>
    */
   List l = new ArrayList();

   public DummyTransaction(DummyTransactionManager tm)
   {
      tm_ = tm;
      status = Status.STATUS_ACTIVE;
   }

   /**
    * Attempt to commit this transaction.
    *
    * @throws RollbackException          If the transaction was marked for rollback
    *                                    only, the transaction is rolled back and this exception is
    *                                    thrown.
    * @throws SystemException            If the transaction service fails in an
    *                                    unexpected way.
    * @throws HeuristicMixedException    If a heuristic decision was made and
    *                                    some some parts of the transaction have been committed while
    *                                    other parts have been rolled back.
    * @throws HeuristicRollbackException If a heuristic decision to roll
    *                                    back the transaction was made.
    * @throws SecurityException          If the caller is not allowed to commit this
    *                                    transaction.
    */
   public void commit()
      throws RollbackException, HeuristicMixedException,
      HeuristicRollbackException, SecurityException, SystemException
   {

      status = Status.STATUS_COMMITTING;
      try
      {
         notifyBeforeCompletion();
         status = Status.STATUS_COMMITTED;
         notifyAfterCompletion(status);
      }
      catch (Throwable t)
      {
         status = Status.STATUS_ROLLEDBACK;
         throw new RollbackException(t.getMessage());
      }

      finally {
          // Disassociate tx from thread.
         tm_.setTransaction(null);
      }
   }

   /**
    * Rolls back this transaction.
    *
    * @throws IllegalStateException If the transaction is in a state
    *                               where it cannot be rolled back. This could be because the
    *                               transaction is no longer active, or because it is in the
    *                               {@link Status#STATUS_PREPARED prepared state}.
    * @throws SystemException       If the transaction service fails in an
    *                               unexpected way.
    */
   public void rollback() throws IllegalStateException, SystemException
   {
      status = Status.STATUS_ROLLING_BACK;
      try
      {
         status = Status.STATUS_ROLLEDBACK;
         notifyAfterCompletion(status);
      }
      catch (Throwable t)
      {
         status = Status.STATUS_ROLLEDBACK;
      }

      // Disassociate tx from thread.
      tm_.setTransaction(null);
   }

   /**
    * Mark the transaction so that the only possible outcome is a rollback.
    *
    * @throws IllegalStateException If the transaction is not in an active
    *                               state.
    * @throws SystemException       If the transaction service fails in an
    *                               unexpected way.
    */
   public void setRollbackOnly() throws IllegalStateException, SystemException
   {
      if (status != Status.STATUS_ACTIVE && status != Status.STATUS_PREPARING)
         throw new IllegalStateException("not active");

      status = Status.STATUS_MARKED_ROLLBACK;
   }

   /**
    * Get the status of the transaction.
    *
    * @return The status of the transaction. This is one of the
    *         {@link Status} constants.
    * @throws SystemException If the transaction service fails in an
    *                         unexpected way.
    */
   public int getStatus() throws SystemException
   {
      return status;
   }

   /**
    * Change the transaction timeout for transactions started by the calling
    * thread with the {@link DummyTransactionManager#begin()} method.
    *
    * @param seconds The new timeout value, in seconds. If this parameter
    *                is <code>0</code>, the timeout value is reset to the default
    *                value.
    * @throws SystemException If the transaction service fails in an
    *                         unexpected way.
    */
   public void setTransactionTimeout(int seconds) throws SystemException
   {
      throw new SystemException("not supported");
   }

   /**
    * Enlist an XA resource with this transaction.
    *
    * @return <code>true</code> if the resource could be enlisted with
    *         this transaction, otherwise <code>false</code>.
    * @throws RollbackException     If the transaction is marked for rollback
    *                               only.
    * @throws IllegalStateException If the transaction is in a state
    *                               where resources cannot be enlisted. This could be because the
    *                               transaction is no longer active, or because it is in the
    *                               {@link Status#STATUS_PREPARED prepared state}.
    * @throws SystemException       If the transaction service fails in an
    *                               unexpected way.
    */
   public boolean enlistResource(XAResource xaRes)
      throws RollbackException, IllegalStateException, SystemException
   {
      throw new SystemException("not supported");
   }

   /**
    * Delist an XA resource from this transaction.
    *
    * @return <code>true</code> if the resource could be delisted from
    *         this transaction, otherwise <code>false</code>.
    * @throws IllegalStateException If the transaction is in a state
    *                               where resources cannot be delisted. This could be because the
    *                               transaction is no longer active.
    * @throws SystemException       If the transaction service fails in an
    *                               unexpected way.
    */
   public boolean delistResource(XAResource xaRes, int flag)
      throws IllegalStateException, SystemException
   {
      throw new SystemException("not supported");
   }

   /**
    * Register a {@link Synchronization} callback with this transaction.
    *
    * @throws RollbackException     If the transaction is marked for rollback
    *                               only.
    * @throws IllegalStateException If the transaction is in a state
    *                               where {@link Synchronization} callbacks cannot be registered.
    *                               This could be because the transaction is no longer active,
    *                               or because it is in the
    *                               {@link Status#STATUS_PREPARED prepared state}.
    * @throws SystemException       If the transaction service fails in an
    *                               unexpected way.
    */
   public void registerSynchronization(Synchronization sync)
      throws RollbackException, IllegalStateException, SystemException
   {
      logger_.debug("registering synchronization handler ");
      if (sync != null && !l.contains(sync))
      {
         logger_.debug("registering synchronization handler " + sync);
         l.add(sync);
      }
   }


   void setStatus(int new_status)
   {
      status = new_status;
   }


   void notifyBeforeCompletion()
   {
      for (Iterator it = l.iterator(); it.hasNext();)
      {
         Synchronization s = (Synchronization) it.next();
         logger_.debug("processed beforeCompletion for " + s);
         s.beforeCompletion();
      }
   }

   void notifyAfterCompletion(int status)
   {
      for (Iterator it = l.iterator(); it.hasNext();)
      {
         Synchronization s = (Synchronization) it.next();
         logger_.debug("processed afterCompletion for " + s);
         s.afterCompletion(status);
      }
      l.clear();
   }

}
