/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.mx.server;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.ReflectionException;
import javax.management.ObjectName;

import org.jboss.mx.server.AbstractMBeanInvoker;


/**
 * @author  <a href="mailto:juha@jboss.org">Juha Lindfors</a>.
 * @version $Revision: 1.4.2.4 $
 *   
 */
public class RawDynamicInvoker
   extends AbstractMBeanInvoker
{
   
   private DynamicMBean typedRes = null;
   
   public RawDynamicInvoker(DynamicMBean resource)
   {
      super(resource);
      this.typedRes = resource;
   }

   // DynamicMBean overrides ----------------------------------------
   
   public void setAttribute(Attribute attribute) throws AttributeNotFoundException,
         InvalidAttributeValueException, MBeanException, ReflectionException
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         typedRes.setAttribute(attribute);
      }
      finally
      {
         TCLStack.pop();
      }
   }

   public AttributeList setAttributes(AttributeList attributes)
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         return typedRes.setAttributes(attributes);
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public Object getAttribute(String name) throws AttributeNotFoundException,
         MBeanException, ReflectionException
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         return typedRes.getAttribute(name);
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public AttributeList getAttributes(String[] attributes)
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         return typedRes.getAttributes(attributes);
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public Object invoke(String name, Object[] args, String[] signature) throws
         MBeanException, ReflectionException
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         return typedRes.invoke(name, args, signature);
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public MBeanInfo getMBeanInfo()
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         return typedRes.getMBeanInfo();
      }
      finally
      {
         TCLStack.pop();
      }
   }

   // MBeanRegistration overrides -----------------------------------
   public ObjectName preRegister(MBeanServer server, ObjectName oname) throws Exception
   {
      this.resourceEntry = super.getMBeanEntry();
      this.info = getMBeanInfo();

      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         if (getResource() instanceof MBeanRegistration)
            return ((MBeanRegistration)getResource()).preRegister(server, oname);
         else
            return oname;
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public void postRegister(Boolean b)
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         if (getResource() instanceof MBeanRegistration)
            ((MBeanRegistration)getResource()).postRegister(b);
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public void preDeregister() throws Exception
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         if (getResource() instanceof MBeanRegistration)
            ((MBeanRegistration)getResource()).preDeregister();
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
   public void postDeregister()
   {
      ClassLoader mbeanTCL = resourceEntry.getClassLoader();
      TCLStack.push(mbeanTCL);
      try
      {
         if (getResource() instanceof MBeanRegistration)
            ((MBeanRegistration)getResource()).postDeregister();
      }
      finally
      {
         TCLStack.pop();
      }
   }
   
}
