/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test;

import java.util.ArrayList;
import java.util.Hashtable;

import javax.management.ObjectName;
import javax.naming.InitialContext;

import org.jboss.jmx.adaptor.rmi.RMIAdaptor;

/**
 * Derived implementation of JBossTestServices for cluster testing.
 *
 * @see org.jboss.test.JBossTestServices
 *
 * @author  <a href="mailto:sacha.labourey@cogito-info.ch">Sacha Labourey</a>.
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1.4.4 $
 */
public class JBossTestClusteredServices extends JBossTestServices
{
   protected ArrayList adaptors = null;
   protected ArrayList servers = null;
   protected ArrayList namingURLs = null;
   
   // Constructors --------------------------------------------------
   
   public JBossTestClusteredServices(String className)
   {
      super(className);
   }
   
   RMIAdaptor[] getAdaptors() throws Exception
   {
      init();
      RMIAdaptor[] tmp = new RMIAdaptor[adaptors.size()];
      adaptors.toArray(tmp);
      return tmp;
   }
   String[] getServers() throws Exception
   {
      init();
      String[] tmp = new String[servers.size()];
      servers.toArray(tmp);
      return tmp;
   }
   String[] getNamingURLs() throws Exception
   {
      init();
      String[] tmp = new String[namingURLs.size()];
      namingURLs.toArray(tmp);
      return tmp;
   }

   /** Override to invoke the operation on all servers
    * @param name
    * @param method
    * @param args
    * @param sig
    * @return
    * @throws Exception
    */ 
   protected Object invoke(ObjectName name, String method, Object[] args,
      String[] sig)
      throws Exception
   {
      System.out.println("in invoke!!");
      RMIAdaptor[] adaptors = getAdaptors();

      Object result = null;
      for(int i = 0; i < adaptors.length; i++)
      {
         result = invoke(adaptors[i], name, method, args, sig);
      }

      return result;

   }

   protected void init() throws Exception
   {
      if( initialContext == null )
      {
         initialContext = new InitialContext();
      }
      if( adaptors == null )
      {
         adaptors = new ArrayList();
         servers = new ArrayList();
         namingURLs = new ArrayList();
         String adaptorName = System.getProperty("jbosstest.server.name");
         if( adaptorName == null )
            adaptorName = "jmx/invoker/RMIAdaptor";

         Hashtable env = new Hashtable();
         env.put("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
         env.put("java.naming.factory.url.pkgs", "org.jnp.interfaces");

         // Look for jbosstest.cluster.nodeN properties for the server names
         String node = "jbosstest.cluster.node";
         int count = 0;
         while( count < 10 )
         {
            String prop = node + count;
            String host = System.getProperty(prop);
            count ++;
            if( host == null )
               break;
            log.info(prop+" = "+host);
            servers.add(host);
            // See if there is a jbosstest.cluster.nodeN.jndiurl
            String urlProp = prop + ".jndiurl";
            String urlDefault = "jnp://" + host + ":1099";
            String urlValue = System.getProperty(urlProp, urlDefault);
            namingURLs.add(urlValue);
            env.put("java.naming.provider.url", urlValue);
            // Lookup the adaptor
            InitialContext tmpCtx = new InitialContext(env);
            RMIAdaptor adaptor = (RMIAdaptor) tmpCtx.lookup(adaptorName);
            adaptors.add(adaptor);
         }

         if( adaptors.size() == 0 )
            throw new IllegalStateException("No jbosstest.cluster.node values found");
      }
   }

}
