package org.jboss.test.cache.test.eviction;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.cache.Fqn;
import org.jboss.cache.eviction.Region;
import org.jboss.cache.eviction.RegionManager;
import java.util.HashMap;

/**
 * @author Ben Wang, Feb 11, 2004
 */
public class RegionUnitTestCase extends TestCase
{
   RegionManager regionManager_;
   public RegionUnitTestCase(String s)
   {
      super(s);
   }

   public void setUp() throws Exception
   {
      super.setUp();
      regionManager_ = new RegionManager(null);
      regionManager_.createRegion("/a/b", null);
   }

   public void tearDown() throws Exception
   {
      super.tearDown();
   }

   public void testAddedQueue() {
      Fqn fqn1 = Fqn.fromString("/a/b/c");
      Fqn fqn2 = Fqn.fromString("/a/b/d");
      Fqn fqn3 = Fqn.fromString("/a/b/e");

      Region region = regionManager_.getRegion("/a/b");
      region.setAddedNode(fqn1);
      region.setAddedNode(fqn2);
      region.setAddedNode(fqn3);

      assertEquals("AddedNode queue size ", 3, region.nodeQueueSize());
      Fqn fqn = region.takeLastNode();
      assertEquals("Node retrieved should be FILO ", fqn, fqn1);
      assertEquals("AddedNode queue size ", 2, region.nodeQueueSize());
      fqn = region.takeLastNode();
      fqn = region.takeLastNode();
      fqn = region.takeLastNode();
      assertNull("Node should be null", fqn);
   }

   public void testEventQueue() {
      Fqn fqn1 = Fqn.fromString("/a/b/c");
      Fqn fqn2 = Fqn.fromString("/a/b/d");
      Fqn fqn3 = Fqn.fromString("/a/b/e");

      Region region = regionManager_.getRegion("/a/b");
      region.setRemovedNode(fqn1);
      region.setAddedNode(fqn2);
      region.setVisitedNode(fqn3);

      assertEquals("RemovedNode queue size ", 3, region.nodeQueueSize());
      Integer event = region.takeLastEvent();
      assertEquals("Node retrieved should be FILO ", Region.REMOVE_EVENT, event);
      Fqn fqn = region.takeLastNode();
      fqn = region.takeLastNode();
      fqn = region.takeLastNode();
      fqn = region.takeLastNode();
      assertNull("Node should be null", fqn);
   }

   void log(String msg)
   {
      System.out.println("-- " + msg);
   }

   public static Test suite()
   {
      return new TestSuite(RegionUnitTestCase.class);
   }

   public static void main(String[] args)
   {
      junit.textui.TestRunner.run(suite());
   }

}
