package org.jboss.test.cache.test.generic;

import EDU.oswego.cs.dl.util.concurrent.Sync;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.lock.ReadWriteLockWithUpgrade;

public class ReadWriteLockWithUpgradeUnitTestCase extends TestCase
{
   static final ReadWriteLockWithUpgrade lock_ = new ReadWriteLockWithUpgrade();
   static long SLEEP_MSECS = 500;

   public ReadWriteLockWithUpgradeUnitTestCase(String name)
   {
      super(name);
   }


   public static void main(String[] args) throws Exception
   {
      log("\nBeginning ReadWriteLockWithUpgrade automated testing ...\n");

      junit.textui.TestRunner.run(suite());
   }

   // Needed for JUnit.
   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      // Adding test cases here ...
      suite.addTestSuite(ReadWriteLockWithUpgradeUnitTestCase.class);
      return suite;
   }

   public void setUp()
   {
      log("Setting up test case ...");
   }

   public void tearDown()
   {
      log("Tearing down test case ...");
   }


    /**
     * First thread1 acquires a lock.
     * Then thread2 tries to acquire the lock, but blocks.
     * Thread1 then releases its lock
     * Thread2 proceeds
     * @throws Exception
     */
    public void testReadWriteReadSequence() throws Exception {
        final long TIMEOUT=5000;

        class Thread1 extends Thread {
            public void run() {
                try {
                    System.out.println("(-- Thread1 acquiring lock");
                    lock_.writeLock().attempt(TIMEOUT);
                    System.out.println("(-- Thread1 acquiring lock: done");

                    _sleep(500); // give thread2 time to acquire lock
                    System.out.println("(-- Thread1 releasing lock");
                    lock_.writeLock().release();
                    System.out.println("(-- Thread1 releasing lock: done");
                }
                catch(InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }


        class Thread2 extends Thread {
            public void run() {
                try {
                    System.out.println("(-- Thread2 acquiring lock");
                    lock_.writeLock().attempt(TIMEOUT);
                    System.out.println("(-- Thread2 acquiring lock: done");

                    System.out.println("(-- Thread2 releasing lock");
                    lock_.writeLock().release();
                    System.out.println("(-- Thread2 releasing lock: done");
                }
                catch(InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        Thread1 t1=new Thread1();
        Thread2 t2=new Thread2();

        t1.start();
        _sleep(500);
        t2.start();
        t2.join();
    }

   public void testRead() throws Exception
   {
      Thread t1 = readThread("t1");
      Thread t2 = readThread("t2");

      t1.start();
      t2.start();
      t1.join(3000);
      t2.join(3000);
      if (t1.isAlive() || t2.isAlive())
         fail("Possible deadlock resulted in testRead.");
   }

   protected Thread readThread(String name)
   {
      return new Thread(name)
      {
         public void run()
         {
            try {
               Sync rlock = lock_.readLock();
               rlock.acquire();
               log("acquired read lock");
               _sleep(SLEEP_MSECS);
               rlock.release();
               log("released read lock");
            } catch (Exception ex) {
            }
         }
      };
   }

   protected Thread writeThread(String name)
   {
      return new Thread(name)
      {
         public void run()
         {
            try {
               Sync wlock = null;
               wlock = lock_.writeLock();
               wlock.acquire();
               log("acquired write lock");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released write lock");
            } catch (Exception ex) {
            }
         }
      };
   }

   protected Thread upgradeThread(String name)
   {
      return new Thread(name)
      {
         public void run()
         {
/*
               try {
               Sync wlock = null;
               wlock = lock_.readLock();
               wlock.acquire();
               log("Acquired read lock for upgrade later");
               _sleep(SLEEP_MSECS/2);
               // upgrade lock. Note that read lock will be released
               // and write lock be acquired automatically.
               wlock = lock_.upgradeLock();
               log("Upgraded write lock");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released write lock");
               } catch (Exception ex) {}
*/
         }
      };
   }

   public void testReadWrite() throws Exception
   {
      log("testReadWrite ...");
      Thread t1 = writeThread("t1");
      Thread t2 = writeThread("t2");
      Thread t3 = readThread("t3 read");
      Thread t4 = readThread("t4 read");

      t3.start();
      t1.start();
      t2.start();
      _sleep(SLEEP_MSECS / 2);
      t4.start();
      t1.join(4000);
      t2.join(3000);
      t3.join(2000);
      t4.join(1000);
      if (t1.isAlive() || t2.isAlive() || t3.isAlive() || t4.isAlive())
         fail("Possible deadlock resulted in testReadWrite.");
   }


   public void XtestReadWriteWithUpgrade1() throws Exception
   {
      log("testReadWriteWithUpgrade ...");
      Thread t1 = writeThread("t1");
      Thread t2 = writeThread("t2");
      Thread t4 = upgradeThread("t4");
      Thread t3 = readThread("t3 read");

      t3.start();
      t4.start();
      t2.start();
      t1.start();
      t1.join(10000);
      t2.join(3000);
      t3.join(2000);
      t4.join(1000);
      if (t1.isAlive() || t2.isAlive() || t3.isAlive() || t4.isAlive())
         fail("Possible deadlock resulted in testReadWriteWithUpgrade1.");
   }

   public void XtestReadWriteWithUpgrade2() throws Exception
   {
      log("testReadWriteWithUpgrade ...");
      Thread t1 = writeThread("t1");
      Thread t2 = writeThread("t2");
      Thread t4 = upgradeThread("t4");
      Thread t3 = readThread("t3 read");

      t1.start();
      t3.start();
      t4.start();
      t2.start();
      t1.join(10000);
      t2.join(3000);
      t3.join(2000);
      t4.join(1000);
      if (t1.isAlive() || t2.isAlive() || t3.isAlive() || t4.isAlive())
         fail("Possible deadlock resulted in testReadWriteWithUpgrade2.");
   }

   protected Thread readAttemptThread(String name, final long msecs)
   {
      return new Thread(name)
      {
         public void run()
         {
            try {
               Sync rlock = lock_.readLock();
               if (!rlock.attempt(msecs)) {
                  log("Read lock attempt failed.");
                  fail("Read lock attempt failed.");
                  return;
               }
               log("acquired read lock");
               _sleep(SLEEP_MSECS);
               rlock.release();
               log("released read lock");
            } catch (Exception ex) {
            }
         }
      };
   }

   protected Thread writeAttemptThread(String name, final long msecs)
   {
      return new Thread(name)
      {
         public void run()
         {
            try {
               Sync wlock = lock_.writeLock();
               if (!wlock.attempt(msecs)) {
                  log("Write lock attempt failed.");
                  fail("Write lock attempt failed.");
                  return;
               }
               log("acquired write lock");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released write lock");
            } catch (Exception ex) {
            }
         }
      };
   }

   protected Thread upgradeAttemptThread(String name, final long msecs)
   {
      return new Thread(name)
      {
         public void run()
         {
            try {
               Sync wlock = null;
               wlock = lock_.readLock();
               if (!wlock.attempt(msecs)) {
                  log("Read lock attempt failed.");
                  fail("Read lock attempt failed.");
                  return;
               }
               log("Acquired read lock for upgrade later");
               _sleep(SLEEP_MSECS / 2);
               // upgrade lock. Note that read lock will be released
               // and write lock be acquired automatically.
               wlock = lock_.upgradeLockAttempt(msecs);
               log("Upgraded write lock");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released write lock");
            } catch (Exception ex) {
            }
         }
      };
   }

   public void testReadWriteAttemp() throws Exception
   {
      log("testReadWriteAttemp() ...");
      Thread t1 = new Thread("t1")
      {
         public void run()
         {
            try {
               Sync wlock = lock_.writeLock();
               if (!wlock.attempt(500)) {
                  log("Expected write lock attempt to fail.");
                  return;
               }
               log("acquired");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released");
            } catch (Exception ex) {
            }
         }
      };

      final long msecs = 5000;
      Thread t2 = writeAttemptThread("t2", msecs);
      Thread t3 = readAttemptThread("t3", msecs);

      t3.start();
      t1.start();
      t2.start();
      t1.join();
      t2.join();
      t3.join();
   }

   public void testReadWriteAttempNoWait() throws Exception
   {
      log("testReadWriteAttempNoWait() ...");
      Thread t1 = new Thread("t1")
      {
         public void run()
         {
            try {
               Sync wlock = lock_.writeLock();
               if (!wlock.attempt(0)) {
                  log("Expected write lock attempt to fail.");
                  return;
               }
               log("acquired");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released");
            } catch (Exception ex) {
            }
         }
      };

      final long msecs = 500;
      Thread t2 = writeAttemptThread("t2", msecs);
      Thread t3 = readAttemptThread("t3", msecs);

      t3.start();
      t1.start();
      t2.start();
      t1.join();
      t2.join();
      t3.join();
   }

   public void testReadWriteAttempNoWait1() throws Exception
   {
      log("testReadWriteAttempNoWait1() ...");
      Thread t1 = new Thread("t1")
      {
         public void run()
         {
            try {
               Sync wlock = lock_.writeLock();
               if (!wlock.attempt(0)) {
                  log("Expected write lock attempt to fail.");
                  return;
               }
               log("acquired upgraded writer lock");
               _sleep(SLEEP_MSECS);
               wlock.release();
               log("released");
            } catch (Exception ex) {
            }
         }
      };

      final long msecs = 500;
      Thread t2 = writeAttemptThread("t2", msecs);
      final long msecs1 = 1500;
      Thread t3 = readAttemptThread("t3", msecs1);

      t1.start();
      t2.start();
      t3.start();
      t1.join();
      t2.join();
      t3.join();
   }

   public static void _sleep(long msecs)
   {
      try {
         Thread.sleep(msecs);
      } catch (InterruptedException ex) {
      }
   }

   public static void log(String str)
   {
      System.out.println(Thread.currentThread() + ": "
            + java.util.Calendar.getInstance().getTime() + " : " + str);
   }
}

