/*
 *
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.cache.test.local;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.TreeCache;
import org.jboss.cache.lock.IsolationLevel;
import org.jboss.logging.Logger;

import java.util.Set;

/**
 * Unit test for local TreeCache. Use locking and multiple threads to test
 * concurrent access to the tree.
 *
 * @version $Revision: 1.5.2.2 $
 */
public class ConcurrentUnitTestCase extends TestCase
{
   TreeCache cache;
   private Logger logger_ = Logger.getLogger(ConcurrentUnitTestCase.class);

   public ConcurrentUnitTestCase(String name)
   {
      super(name);
   }

   public void setUp() throws Exception
   {
      super.setUp();
   }

   private void createCache(IsolationLevel level) throws Exception
   {
      cache = new TreeCache();
      cache.setCacheMode(TreeCache.LOCAL);
      cache.setIsolationLevel(level);
      cache.start();
   }

   public void tearDown() throws Exception
   {
      super.tearDown();
      cache.stop();
   }

   public void XtestDummy()
   {
   }

   public void testConcurrentAccessWithRWLock() throws Exception
   {
      createCache(IsolationLevel.REPEATABLE_READ);
      work_();
   }

   public void testConcurrentAccessWithSimpleLock() throws Exception
   {
      createCache(IsolationLevel.SERIALIZABLE);
      work_();
   }

   private void work_()
   {
      Thread one, two;
      try {
         one = new Thread()
         {
            public void run()
            {
               try {
                  log("put(\"/a/b/c)\"");
                  cache.put("/a/b/c", null);
                  log("adding data");
                  for (int i = 0; i < 1000; i++) {
                     log("adding data i=" + i);
                     cache.put("/a/b/c", "number #" + i, "Thread One");
                     yield();
                  }
               } catch (Throwable t) {
                  t.printStackTrace();
                  fail(t.toString());
               }
            }
         };

         two = new Thread()
         {
            public void run()
            {
               try {
                  log("put(\"/a/b/c)\"");
                  cache.put("/a/b/c", null);
                  log("adding data");
                  for (int i = 0; i < 1000; i++) {
                     log("adding data i=" + i);
                     cache.put("/a/b/c", "number #" + i, "Thread Two");
                     yield();
                  }
               } catch (Throwable t) {
                  t.printStackTrace();
                  fail(t.toString());
               }
            }
         };

         one.start();
         two.start();
         one.join();
         two.join();

         System.out.println("cache content: " + cache.toString());
         Set keys = cache.getKeys("/a/b/c");
         log("number of keys=" + keys.size());
         assertEquals(keys.size(), 1000);

//            for(Iterator it=keys.iterator(); it.hasNext();) {
//                Object key=it.next();
//                System.out.println(key + ": " + cache.get("/a/b/c", (String)key));
//            }
         log("lock info:\n" + cache.printLockInfo());

      } catch (Exception e) {
         e.printStackTrace();
         fail(e.toString());
      }
   }


   void sleep(long timeout)
   {
      try {
         Thread.sleep(timeout);
      } catch (InterruptedException e) {
      }
   }

   void log(String msg)
   {
//        System.out.println("-- [" + Thread.currentThread() + "]: " + msg);
      logger_.debug(" [" + Thread.currentThread() + "]: " + msg);
   }


   public static Test suite()
   {
      return new TestSuite(ConcurrentUnitTestCase.class);
   }

   public static void main(String[] args)
   {
      junit.textui.TestRunner.run(suite());
   }


}
