/*
* JBoss, the OpenSource J2EE webOS
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.test.cluster.test;

import java.net.HttpURLConnection;

import junit.framework.Test;

import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.methods.GetMethod;
import org.jboss.test.JBossClusteredTestCase;

/** Tests of http session replication
 *
 * @author  Scott.Stark@jboss.org
 * @version $Revision: 1.1.2.1 $
 */
public class WebSessionTestCase extends JBossClusteredTestCase
{
   public WebSessionTestCase(String name)
   {
      super(name);
   }

   public static Test suite() throws Exception
   {
      Test t1 = getDeploySetup(WebSessionTestCase.class, "dist-ss.war");
      return t1;
   }

   public void testServletSession()
      throws Exception
   {
      getLog().debug("+++ testServletSession");

      String[] servers = getServers();
      String baseURL0 = "http://"+servers[0]+":" + Integer.getInteger("web.port", 8080) + "/"; 
      String baseURL1 = baseURL0;
      if( servers.length > 1 )
         baseURL1 = "http://"+servers[1]+":" + Integer.getInteger("web.port", 8080) + "/";

      // Access the StatefulSessionServlet of dist-ss.war@server0 twice
      HttpClient httpConn = new HttpClient();
      GetMethod servletGet0 = new GetMethod(baseURL0+"dist-ss/StatefulSessionServlet/");
      int responseCode = httpConn.executeMethod(servletGet0);
      assertTrue("Get OK", responseCode == HttpURLConnection.HTTP_OK);
      Header accessCount = servletGet0.getResponseHeader("X-AccessCount");
      int count = Integer.parseInt(accessCount.getValue());
      assertTrue("X-AccessCount == 1", count == 1);
      // Get the state for the JSESSIONID
      HttpState state = httpConn.getState();
      servletGet0 = new GetMethod(baseURL0+"dist-ss/StatefulSessionServlet/");
      responseCode = httpConn.executeMethod(servletGet0.getHostConfiguration(),
         servletGet0, state);
      accessCount = servletGet0.getResponseHeader("X-AccessCount");
      count = Integer.parseInt(accessCount.getValue());
      assertTrue("X-AccessCount == 2", count == 2);
      // Get the JSESSIONID so we can reset the host
      Cookie[] cookies = state.getCookies();
      Cookie sessionID = null;
      for(int c = 0; c < cookies.length; c ++)
      {
         Cookie k = cookies[c];
         if( k.getName().equalsIgnoreCase("JSESSIONID") )
            sessionID = k;
      }
      log.info("Saw JSESSIONID="+sessionID);
      // Reset the domain so that the cookie will be sent to server1
      sessionID.setDomain(servers[1]);
      state.addCookie(sessionID);

      // Access the StatefulSessionServlet of dist-ss.war@server1 twice
      GetMethod servletGet1 = new GetMethod(baseURL1+"dist-ss/StatefulSessionServlet/");
      responseCode = httpConn.executeMethod(servletGet1.getHostConfiguration(),
         servletGet1, state);
      assertTrue("Get OK", responseCode == HttpURLConnection.HTTP_OK);
      accessCount = servletGet1.getResponseHeader("X-AccessCount");
      count = Integer.parseInt(accessCount.getValue());
      assertTrue("X-AccessCount == 3", count == 3);
      servletGet1 = new GetMethod(baseURL1+"dist-ss/StatefulSessionServlet/");
      responseCode = httpConn.executeMethod(servletGet1.getHostConfiguration(),
         servletGet1, state);
      assertTrue("Get OK", responseCode == HttpURLConnection.HTTP_OK);
      accessCount = servletGet1.getResponseHeader("X-AccessCount");
      count = Integer.parseInt(accessCount.getValue());
      assertTrue("X-AccessCount == 4", count == 4);
   }

}
