/*
 * JBoss, the OpenSource WebOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.web.tomcat.tc5;

import org.jboss.web.AbstractWebContainerMBean;

/** 
 * An implementation of the AbstractWebContainer for the Jakarta Tomcat5
 * servlet container. It has no code dependency on tomcat - only the new JMX
 * model is used.
 * 
 * Tomcat5 is organized as a set of mbeans - just like jboss.
 * 
 * @see AbstractWebContainerMBean
 * 
 * @author Scott.Stark@jboss.org
 * @author Costin Manolache
 * @version $Revision: 1.1.1.1.2.7 $
 */
public interface Tomcat5MBean extends AbstractWebContainerMBean
{

    public String getDomain();

    /** The most important attribute - defines the managed domain.
     *  A catalina instance (engine) corresponds to a JMX domain, that's
     *  how we know where to deploy webapps.
     *
     * @param domainName the jmx domain under which tc registers
     */
    public void setDomain(String domainName);

   /** Set the snapshot mode in a clustered environment */
   public void setSnapshotMode(String mode);

   /** Get the snapshot mode in a clustered environment */
   public String getSnapshotMode();

   /** Set the snapshot interval in ms for the interval snapshot mode */
   public void setSnapshotInterval(int interval);

   /** Get the snapshot interval */
   public int getSnapshotInterval();

   /** Get the clustering code cache behaviour */
   public boolean isUseLocalCache();

   /** Set the clustering code cache behaviour */
   public void setUseLocalCache(boolean useLocalCache);

   /**
    * Gets the JMX object name of a shared TreeCache to be used for clustered 
    * single-sign-on.
    * 
    * @see org.jboss.web.tomcat.tc5.sso.TreeCacheSSOClusterManager 
    */      
   public String getCacheName();

   /**
    * Gets the JMX object name of a shared TreeCache to be used for clustered 
    * single-sign-on.
    * <p>
    * <b>NOTE:</b> TreeCache must be deployed before this service.
    * 
    * @see org.jboss.web.tomcat.tc5.sso.TreeCacheSSOClusterManager 
    */      
   public void setCacheName(String cacheName);

   /** Get the JBoss UCL use flag */
   public boolean getUseJBossWebLoader();
   /** Set the JBoss UCL use flag */
   public void setUseJBossWebLoader(boolean flag);

   public String getManagerClass();

   public void setManagerClass(String managerClass);

   /** */
   public String getContextMBeanCode();
   /** */
   public void setContextMBeanCode(String className);

   /** Get the name of the external tomcat server configuration file.
    * 
    * @return the config file name, server.xml for example
    */ 
   public String getConfigFile();
   /** Set the name of the external tomcat server configuration file.
    * 
    * @param configFile - the config file name, server.xml for example
    */ 
   public void setConfigFile(String configFile);

   /** Get the request attribute name under which the JAAS Subject is store */
   public String getSubjectAttributeName();
   /** Set the request attribute name under which the JAAS Subject will be
    * stored when running with a security mgr that supports JAAS. If this is
    * empty then the Subject will not be store in the request.
    * @param name the HttpServletRequest attribute name to store the Subject
    */
   public void setSubjectAttributeName(String name);

   /** Start all connectors of the Domain + ":type=Service,serviceName=jboss.web"
    * service.
    * 
    * @throws Exception
    */ 
   public void startConnectors() throws Exception;
   /** Stop all connectors of the Domain + ":type=Service,serviceName=jboss.web"
    * service.
    * 
    * @throws Exception
    */ 
   public void stopConnectors() throws Exception;   
}
